Provides a field widget for file-fields that uses the vmsg HTML5/JavaScript
library for in-browser recording.

See Middlebury's [name-pronunciation documentation](https://mediawiki.middlebury.edu/LIS/CourseHub_name_pronunciation)
for an example of how this field widget can be used.

Installation and Setup
======================

1. Update your project's root `composer.json` to support the installation of
   library dependencies from asset-packagist to the correct location in your
   Drupal file-structure.

   *  Add `https://asset-packagist.org` as a `composer` repository to your
      project's root `composer.json` similar to this (`repositories` can either be
      an array or a map):

      ```
      "repositories": {
         "drupal": {
             "type": "composer",
             "url": "https://packages.drupal.org/8"
         },
      +   "asset-packagist": {
      +       "type": "composer",
      +       "url": "https://asset-packagist.org"
      +   }
      }
      ```

      According to the [Composer documentation](https://getcomposer.org/doc/05-repositories.md):

      > "Repositories are only available to the root package and the repositories
      > defined in your dependencies will not be loaded."

   *  Require the `oomphinc/composer-installers-extender` package which provides
      the ability to define install-paths for types of dependencies as described
      in [Using Composer to Install Drupal and Manage Dependencies --> Downloading third-party libraries using Composer](https://www.drupal.org/docs/develop/using-composer/using-composer-to-install-drupal-and-manage-dependencies#third-party-libraries).

      In your project's root `composer.json` add lines under `extra` that define
      where you want the libraries to be installed:

      ```
      "extra": {

      +   "installer-types": [
      +       "npm-asset",
      +       "bower-asset"
      +   ],
          "installer-paths": {
              "web/core": ["type:drupal-core"],
              "web/libraries/{$name}": [
                "type:drupal-library",
      +         "type:npm-asset",
      +         "type:bower-asset"
              ]
              "web/modules/contrib/{$name}": ["type:drupal-module"],
              "web/modules/custom/{$name}": ["type:drupal-custom-module"],
              "web/profiles/contrib/{$name}": ["type:drupal-profile"],
              "web/themes/contrib/{$name}": ["type:drupal-theme"],
              "web/themes/custom/{$name}": ["type:drupal-custom-theme"]
          },
      },
      ```

      Note that your project *may* use a different directory structure and
      and webserver configuration. The important thing is that the vmsg
      libraries get installed along-side other Drupal libraries so that they
      can be properly located by

   *  Add accessability patches to vmsg (as long as `vmsg 0.3.6` is the latest
      release available). Once a new release of vmsg is available this will no
      longer be needed.

      Require the `cweagans/composer-patches` package which provides
      the ability to patch dependencies as described
      in [Using Composer to Install Drupal and Manage Dependencies --> Patching projects using Composer](https://www.drupal.org/docs/develop/using-composer/using-composer-to-install-drupal-and-manage-dependencies#patches).

      In your project's root `composer.json` add lines under `patches`:

      ```
      "patches": {

      +   "npm-asset/vmsg": {
      +       "Improve accessibility": "https://github.com/middlebury/vmsg/commit/3f0ddbbc7ce5566bba641c709bf7bc74b237361b.diff"
      +   }

      }
      ```

      See [Issue #3197356](https://www.drupal.org/project/audiorecorder/issues/3197356) for details and status.

2. Add the module and its vmsg dependency with composer in your Drupal project:
   `composer require drupal/audiorecorder:^1.0`.


3. The vmsg library uses web-assembly, so you must tell your webserver to use the
   appropriate content-type for `.wasm` files in your `.htaccess`:

   ```
   # Add correct encoding for Web-Assembly.
   AddType application/wasm wasm
   ```

4. If you find that your composer workflow is resulting in the file-permissions
   that are preventing your webserver from reading the `vmsg.js` and `vmsg.wasm`
   files, you can address this by adding something like the following to your
   project's root `composer.json`. This generally should be safe, but in your
   server environment it may be preferable to change read access only for your
   webserver user. See [Issue #3197357](https://www.drupal.org/project/audiorecorder/issues/3197357)
   for more details.

   ```
   "scripts": {
       "post-install-cmd": [
           "chmod a+r web/libraries/vmsg/vmsg.*"
       ]
   },
   ```

5. Activate the module.

6. Add a file field to an entity. Set the "Allowed file extensions" to `mp3`.
   Set a reasonable maximum upload size as browsers that don't support the
   audio-recording APIs will display a file-upload widget instead of the
   recording UI.

7. In the "Manage Form Settings" tab for the entity, choose "Audio Recorder" as
   the widget for your file field. You can also set a maximum recording time.


Troubleshooting
===============

Browser can't load libraries
---------------------------------------

If you find that your composer workflow is resulting in the file-permissions
that are preventing your webserver from reading the `vmsg.js` and `vmsg.wasm`
files, you can address this by adding something like the following to your
project's root `composer.json` then re-running `composer install`.

```
    "scripts": {
        "post-install-cmd": [
            "chmod a+r web/libraries/vmsg/vmsg.*"
        ]
    },
```
This generally should be safe, but in your server environment it may be
preferable to change read access only for your webserver user.

See [Issue #3197357](https://www.drupal.org/project/audiorecorder/issues/3197357)
for more details.
